@@ -1,4 +1,4 @@
-# Copyright 2022 Google LLC.
+# Copyright 2023 Google LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,6 +15,10 @@
 """Plotting functions pre and post model fitting."""
 
 import functools
+import logging
+
+# Using these types from typing instead of their generic types in the type hints
+# in order to be compatible with Python 3.7 and 3.8.
 from typing import Any, List, Optional, Sequence, Tuple
 
 import arviz
@@ -23,12 +27,17 @@ import jax.numpy as jnp
 import matplotlib
 import matplotlib.pyplot as plt
 import numpy as np
+import numpyro
 import pandas as pd
 import seaborn as sns
 from sklearn import metrics
 
 from lightweight_mmm import lightweight_mmm
+from lightweight_mmm import models
 from lightweight_mmm import preprocessing
+from lightweight_mmm import utils
+
+plt.style.use("default")
 
 _PALETTE = sns.color_palette(n_colors=100)
 
@@ -132,6 +141,113 @@ def _calculate_number_rows_plot(n_media_channels: int, n_columns: int):
   return n_media_channels // n_columns + 2
 
 
+def _calculate_media_contribution(
+    media_mix_model: lightweight_mmm.LightweightMMM) -> jnp.ndarray:
+  """Computes contribution for each sample, time, channel.
+
+  Serves as a helper function for making predictions for each channel, time
+  and estimate sample. It is meant to be used in creating media baseline
+  contribution dataframe and visualize media attribution over spend proportion
+  plot.
+
+  Args:
+    media_mix_model: Media mix model.
+
+  Returns:
+    Estimation of contribution for each sample, time, channel.
+
+  Raises:
+    NotFittedModelError: if the model is not fitted before computation
+  """
+  if not hasattr(media_mix_model, "trace"):
+    raise lightweight_mmm.NotFittedModelError(
+        "Model needs to be fit first before attempting to plot its fit.")
+
+  if media_mix_model.trace["media_transformed"].ndim > 3:
+    # s for samples, t for time, c for media channels, g for geo
+    einsum_str = "stcg, scg->stcg"
+  elif media_mix_model.trace["media_transformed"].ndim == 3:
+    # s for samples, t for time, c for media channels
+    einsum_str = "stc, sc->stc"
+
+  media_contribution = jnp.einsum(einsum_str,
+                                  media_mix_model.trace["media_transformed"],
+                                  media_mix_model.trace["coef_media"])
+  if media_mix_model.trace["media_transformed"].ndim > 3:
+    # Aggregate media channel contribution across geos.
+    media_contribution = media_contribution.sum(axis=-1)
+  return media_contribution
+
+
+def create_attribution_over_spend_fractions(
+    media_mix_model: lightweight_mmm.LightweightMMM,
+    media_spend: jnp.ndarray,
+    channel_names: Optional[Sequence[str]] = None,
+    time_index: Optional[Tuple[int, int]] = None) -> pd.DataFrame:
+  """Creates a dataframe for media attribution over spend.
+
+  The output dataframe will be used to create media attribution and spend
+  barplot; and attribution over spend lineplot
+
+  Args:
+    media_mix_model: Media mix model.
+    media_spend: Media spend per channel. If 1D, it needs to be pre-processed to
+      align with time index range. 2D represents media spend per time and
+      channel. Media spends need to be aggregated over geo before input.
+    channel_names: Names of media channels to be added to the output dataframe.
+    time_index: Time range index used for calculation.
+
+  Returns:
+    DataFrame containing fractions of the contribution and spend and
+    attribution over spend for each channel.
+
+  Rasies:
+    ValueError: if any of the media values are negative or any aggregated media
+    spends are zero or negative.
+    NotFittedModelError: if the model is not fitted.
+  """
+  if (media_spend < 0).any():
+    raise ValueError("Values in media must all be non-negative or values "
+                     "in aggregated media must be possitive.")
+  if channel_names is None:
+    try:
+      channel_names = media_mix_model.media_names
+    except AttributeError as att_error:
+      raise lightweight_mmm.NotFittedModelError(
+          "Model needs to be fit first before attempting to plot its fit."
+      ) from att_error
+
+  media_contribution = _calculate_media_contribution(media_mix_model)
+  if time_index is None:
+    time_index = (0, media_contribution.shape[1])
+
+  # Select time span and aggregate 2D media spend.
+  if media_spend.ndim == 1:
+    logging.warning("1D media spend has to align with time index range.")
+  elif media_spend.ndim == 2:
+    media_spend = media_spend[time_index[0]:time_index[1], :]
+    media_spend = media_spend.sum(axis=0)
+
+  # Select time span and aggregate media contribution.
+  media_contribution = media_contribution[:, time_index[0]:time_index[1],]
+  media_contribution = media_contribution.sum(axis=(0, 1))
+
+  # Create media contribution and spend dataframe
+  media_df = pd.DataFrame(
+      np.transpose([media_contribution, media_spend]),
+      index=channel_names,
+      columns=["media attribution", "media spend"])
+
+  if (media_df["media spend"] == 0).any():
+    raise ValueError("Values in media must all be non-negative or values "
+                     "in aggregated media must be possitive.")
+
+  normalized_media_df = media_df.div(media_df.sum(axis=0), axis=1)
+  normalized_media_df["attribution over spend"] = normalized_media_df[
+      "media attribution"] / normalized_media_df["media spend"]
+  return normalized_media_df
+
+
 def create_media_baseline_contribution_df(
     media_mix_model: lightweight_mmm.LightweightMMM,
     target_scaler: Optional[preprocessing.CustomScaler] = None,
@@ -149,28 +265,9 @@ def create_media_baseline_contribution_df(
   Returns:
     contribution_df: DataFrame of weekly channels & baseline contribution
     percentage & volume.
-
-  Raises:
-    NotFittedModelError: If the model is not fitted before visualization.
   """
-  if not hasattr(media_mix_model, "trace"):
-    raise lightweight_mmm.NotFittedModelError(
-        "Model needs to be fit first before attempting to plot its fit.")
-
-  if media_mix_model.trace["media_transformed"].ndim > 3:
-    # s for samples, t for time, c for media channels, g for geo
-    einsum_str = "stcg, scg->stcg"
-  elif media_mix_model.trace["media_transformed"].ndim == 3:
-    # s for samples, t for time, c for media channels
-    einsum_str = "stc, sc->stc"
-
-  scaled_media_contribution = jnp.einsum(
-      einsum_str, media_mix_model.trace["media_transformed"],
-      media_mix_model.trace["coef_media"])
-
-  if media_mix_model.trace["media_transformed"].ndim > 3:
-    # Aggregate media channel contribution across geos.
-    scaled_media_contribution = scaled_media_contribution.sum(axis=-1)
+  # Create media contribution matrix.
+  scaled_media_contribution = _calculate_media_contribution(media_mix_model)
 
   # Aggregate media channel contribution across samples.
   sum_scaled_media_contribution_across_samples = scaled_media_contribution.sum(
@@ -204,7 +301,14 @@ def create_media_baseline_contribution_df(
   media_contribution_pct_by_channel = (
       sum_scaled_media_contribution_across_samples /
       adjusted_sum_scaled_prediction_across_samples.reshape(-1, 1))
+  # Adjust media pct contribution if the value is nan
+  media_contribution_pct_by_channel = np.nan_to_num(
+      media_contribution_pct_by_channel)
+
   baseline_contribution_pct = adjusted_sum_scaled_baseline_contribution_across_samples / adjusted_sum_scaled_prediction_across_samples
+  # Adjust baseline pct contribution if the value is nan
+  baseline_contribution_pct = np.nan_to_num(
+      baseline_contribution_pct)
 
   # If the channel_names is none, then create naming covention for the channels.
   if channel_names is None:
@@ -233,6 +337,12 @@ def create_media_baseline_contribution_df(
   # Take the average of the inverse transformed prediction across samples.
   posterior_pred_df = pd.DataFrame(
       posterior_pred.mean(axis=0), columns=["avg_prediction"])
+
+  # Adjust prediction value when prediction is less than 0.
+  posterior_pred_df["avg_prediction"] = np.where(
+      posterior_pred_df["avg_prediction"] < 0, 0,
+      posterior_pred_df["avg_prediction"])
+
   contribution_pct_df.columns = [
       "{}_percentage".format(col) for col in contribution_pct_df.columns
   ]
@@ -264,7 +374,7 @@ def plot_response_curves(
     steps: int = 50,
     percentage_add: float = 0.2,
     apply_log_scale: bool = False,
-    figure_size: Tuple[int, int] = (10, 14),
+    figure_size: Tuple[int, int] = (8, 10),
     n_columns: int = 3,
     marker_size: int = 8,
     legend_fontsize: int = 8,
@@ -827,7 +937,7 @@ def plot_pre_post_budget_allocation_comparison(
       x_axis + 0.2,
       optimal_buget_allocation,
       width=0.4,
-      label="optmized buget allocation")
+      label="optimized budget allocation")
   axes[0].set_ylabel("Budget Allocation", fontsize="x-large")
   axes[0].set_title(
       "Before and After Optimization Budget Allocation Comparison",
@@ -861,7 +971,7 @@ def plot_pre_post_budget_allocation_comparison(
   axes[0].legend(fontsize="medium")
 
   plots3 = axes[1].bar([
-      "pre optimization predictited target",
+      "pre optimization predicted target",
       "post optimization predicted target"
   ], predictions)
   axes[1].set_ylim(
@@ -898,7 +1008,8 @@ def plot_media_baseline_contribution_area_plot(
     media_mix_model: lightweight_mmm.LightweightMMM,
     target_scaler: Optional[preprocessing.CustomScaler] = None,
     channel_names: Optional[Sequence[Any]] = None,
-    fig_size: Optional[Tuple[int, int]] = (30, 10)
+    fig_size: Optional[Tuple[int, int]] = (20, 7),
+    legend_outside: Optional[bool] = False,
 ) -> matplotlib.figure.Figure:
   """Plots an area chart to visualize weekly media & baseline contribution.
 
@@ -907,6 +1018,7 @@ def plot_media_baseline_contribution_area_plot(
     target_scaler: Scaler used for scaling the target.
     channel_names: Names of media channels.
     fig_size: Size of the figure to plot as used by matplotlib.
+    legend_outside: Put the legend outside of the chart, center-right.
 
   Returns:
     Stacked area chart of weekly baseline & media contribution.
@@ -938,7 +1050,385 @@ def plot_media_baseline_contribution_area_plot(
   ax.set_xlim(1, contribution_df_for_plot["period"].max())
   ax.set_xticks(contribution_df_for_plot["period"])
   ax.set_xticklabels(contribution_df_for_plot["period"])
+  # Get handles and labels for sorting.
+  handles, labels = ax.get_legend_handles_labels()
+  # If true, legend_outside reversed the legend and puts the legend center left,
+  # outside the chart.
+  # If false, legend_outside only reverses the legend order.
+
+  # Channel order is based on the media input and chart logic.
+  # Chart logic puts the last column onto the bottom.
+  # To be in line with chart order, we reserve the channel order in the legend.
+  if legend_outside:
+    ax.legend(handles[::-1], labels[::-1],
+              loc="center left", bbox_to_anchor=(1, 0.5))
+  # Only sort the legend.
+  else:
+    ax.legend(handles[::-1], labels[::-1])
+
   for tick in ax.get_xticklabels():
     tick.set_rotation(45)
   plt.close()
   return fig
+
+
+def _make_prior_and_posterior_subplot_for_one_feature(
+    prior_distribution: numpyro.distributions.Distribution,
+    posterior_samples: np.ndarray,
+    subplot_title: str,
+    fig: matplotlib.figure.Figure,
+    gridspec_fig: matplotlib.gridspec.GridSpec,
+    i_ax: int,
+    hyperprior: bool = False,
+    number_of_samples_for_prior: int = 5000,
+    kde_bandwidth_adjust_for_posterior: float = 1,
+    seed: Optional[int] = None,
+) -> Tuple[matplotlib.figure.Figure, matplotlib.gridspec.GridSpec, int]:
+  """Helper function to make the prior and posterior distribution subplots.
+
+  This function makes some (hard-coded) choices about how to display the prior
+  and posterior distributions. First, it uses kernel density estimators to
+  smooth the distributions rather than plotting the histograms directly (since
+  the histograms look too noisy unless we take unreasonably large numbers of
+  samples). Second, we found that the default bandwidth works pretty well for
+  visualization of these distributions, though we expose the bw_adjust parameter
+  if users wish to modify this. Finally, kernel density estimators tend to have
+  issues at the edges of distributions, and this issue persists here too. We
+  clip some distributions (Half-Normal, Beta, Gamma) to keep the KDE plot
+  representations inside the domains where these distributions are defined, and
+  we also set cut=0 for the posterior distributions to give better insight into
+  their exact ranges. We don't do this for the prior distributions, since in
+  general they have wider (or infinite) ranges and the bounds that we show would
+  be mostly determined by number_of_samples_for_prior, which should just be an
+  implementation detail.
+
+  Args:
+    prior_distribution: Numpyro distribution specifying the prior from which we
+      will sample to generate the plot.
+    posterior_samples: Array of samples from the posterior distribution,
+      obtained from the trace of the media_mix_model. Might need to be flattened
+      in some cases.
+    subplot_title: Title to display for this particular subplot
+    fig: The matplotlib Figure object for the overall plot.
+    gridspec_fig: The matplotlib GridSpec object for the overall plot.
+    i_ax: Index of the subplot within the gridspec_fig.
+    hyperprior: Flag which indicates that the prior_distribution is actually a
+      hyperprior distribution. LMMM is hierarchical on the channel coefficients
+      when run at the geo-level, so this should currently only be set to True
+      when working with the media channel coefficients.
+    number_of_samples_for_prior: Controls the level of smoothing for the plotted
+      version of the prior distribution. The default should be fine unless you
+      want to decrease it to speed up runtime.
+    kde_bandwidth_adjust_for_posterior: Multiplicative factor to adjust the
+      bandwidth of the kernel density estimator, to control the level of
+      smoothing for the posterior distribution. Passed to seaborn.kdeplot as the
+      bw_adjust parameter there.
+    seed: Seed to use for PRNGKey during sampling. For replicability run
+        this function and any other function that utilises predictions with the
+        same seed.
+
+  Returns:
+    fig: The matplotlib Figure object for the overall plot.
+    gridspec_fig: The matplotlib GridSpec object for the overall plot.
+    i_ax: Index of the subplot within the gridspec_fig, iterated by one.
+  """
+
+  if seed is None:
+    seed = utils.get_time_seed()
+  prior_samples = prior_distribution.sample(
+      key=jax.random.PRNGKey(seed=seed),
+      sample_shape=(number_of_samples_for_prior,))
+
+  # Truncate the KDE plot representation of Half-Normal, Beta, and Gamma
+  # distributions, since users might be confused to see a Half-Normal or Gamma
+  # going negative, or a Beta distribution outside of [0, 1].
+  if isinstance(
+      prior_distribution,
+      (numpyro.distributions.HalfNormal, numpyro.distributions.Gamma)):
+    clipping_bounds = [0, None]
+  elif isinstance(prior_distribution, numpyro.distributions.Beta):
+    clipping_bounds = [0, 1]
+  else:
+    clipping_bounds = None
+
+  if hyperprior:
+    square_root_of_number_of_samples_for_prior = int(
+        np.sqrt(number_of_samples_for_prior))
+    prior_distribution = numpyro.distributions.continuous.HalfNormal(
+        scale=prior_samples[:square_root_of_number_of_samples_for_prior])
+    prior_samples = prior_distribution.sample(
+        key=jax.random.PRNGKey(seed=seed),
+        sample_shape=(square_root_of_number_of_samples_for_prior,)).flatten()
+
+  ax = fig.add_subplot(gridspec_fig[i_ax, 0])
+  sns.kdeplot(
+      data=prior_samples,
+      lw=4,
+      clip=clipping_bounds,
+      color="tab:blue", ax=ax, label="prior")
+  prior_xlims = ax.get_xlim()
+
+  sns.kdeplot(
+      data=posterior_samples.flatten(),
+      lw=4,
+      clip=clipping_bounds,
+      cut=0,
+      bw_adjust=kde_bandwidth_adjust_for_posterior,
+      color="tab:orange", ax=ax, label="posterior")
+  posterior_xlims = ax.get_xlim()
+
+  ax.legend(loc="best")
+  ax.set_xlim(
+      min(prior_xlims[0], posterior_xlims[0]),
+      max(prior_xlims[1], posterior_xlims[1]))
+  ax.set_yticks([])
+  ax.set_ylabel("")
+  ax.set_title(subplot_title, y=0.85, va="top", fontsize=10)
+
+  i_ax += 1
+
+  return fig, gridspec_fig, i_ax
+
+
+def _collect_features_for_prior_posterior_plot(
+    media_mix_model: lightweight_mmm.LightweightMMM,
+    selected_features: Optional[List[str]] = None,
+) -> Tuple[List[str], List[str], List[str], List[str], List[str]]:
+  """Helper function to collect features to include in the prior/posterior plot.
+
+  Args:
+    media_mix_model: Fitted media mix model.
+    selected_features: Optional list of feature names to select. If not
+      specified (the default), all features are selected.
+
+  Returns:
+    features: List of all features for the given model type, for which we have
+    defined prior distributions.
+    geo_level_features: List of all geo-level features for the given model type.
+    channel_level_features: List of all channel-level features for the given
+    model type.
+    seasonal_features: List of all seasonal features for the given model type.
+    other_features: List of all other features for the given media_mix_model.
+
+  Raises:
+    ValueError: if feature names are passed to selected_features which do not
+    appear in media_mix_model.
+  """
+
+  media_mix_model_attributes_to_check_for = [
+      "trace",
+      "_weekday_seasonality",
+      "custom_priors",
+      "n_geos",
+      "n_media_channels",
+      "_media_prior",
+  ]
+  if not all([
+      hasattr(media_mix_model, x)
+      for x in media_mix_model_attributes_to_check_for
+  ]):
+    raise lightweight_mmm.NotFittedModelError(
+        "Model needs to be fit first in order to plot the posterior.")
+
+  features = media_mix_model._prior_names
+  if not media_mix_model._weekday_seasonality:
+    features = features.difference([models._WEEKDAY])
+
+  if media_mix_model._extra_features is None:
+    features = features.difference(["coef_extra_features"])
+
+  if media_mix_model.media.ndim == 2:
+    features = features.difference(models.GEO_ONLY_PRIORS)
+    features = features.union(["coef_media"])
+  else:
+    features = features.union(["coef_media", "channel_coef_media"])
+
+  if selected_features:
+    extraneous_features = set(selected_features).difference(features)
+    if extraneous_features:
+      raise ValueError(
+          f"Selected_features {extraneous_features} not in media_mix_model.")
+    features = selected_features
+
+  geo_level_features = [
+      models._COEF_SEASONALITY,
+      models._COEF_TREND,
+      models._INTERCEPT,
+      models._SIGMA,
+  ]
+  channel_level_features = [
+      models._AD_EFFECT_RETENTION_RATE,
+      models._EXPONENT,
+      models._HALF_MAX_EFFECTIVE_CONCENTRATION,
+      models._LAG_WEIGHT,
+      models._PEAK_EFFECT_DELAY,
+      models._SLOPE,
+      "channel_coef_media",
+      "coef_media",
+  ]
+  seasonal_features = [models._GAMMA_SEASONALITY]
+  if media_mix_model._weekday_seasonality:
+    seasonal_features.append(models._WEEKDAY)
+  other_features = list(set(features) - set(geo_level_features) -
+                        set(channel_level_features) - set(seasonal_features))
+
+  return (list(features), geo_level_features, channel_level_features,
+          seasonal_features, other_features)
+
+
+def plot_prior_and_posterior(
+    media_mix_model: lightweight_mmm.LightweightMMM,
+    fig_size: Optional[Tuple[int, int]] = None,
+    selected_features: Optional[List[str]] = None,
+    number_of_samples_for_prior: int = 5000,
+    kde_bandwidth_adjust_for_posterior: float = 1,
+    seed: Optional[int] = None,
+) -> matplotlib.figure.Figure:
+  """Plots prior and posterior distributions for parameters in media_mix_model.
+
+  Args:
+    media_mix_model: Fitted media mix model.
+    fig_size: Size of the figure to plot as used by matplotlib. Default is a
+      width of 8 and a height of 1.5 for each subplot.
+    selected_features: Optional list of feature names to select. If not
+      specified (the default), all features are selected.
+    number_of_samples_for_prior: Controls the level of smoothing for the plotted
+      version of the prior distribution. The default should be fine unless you
+      want to decrease it to speed up runtime.
+    kde_bandwidth_adjust_for_posterior: Multiplicative factor to adjust the
+      bandwidth of the kernel density estimator, to control the level of
+      smoothing for the posterior distribution. Passed to seaborn.kdeplot as the
+      bw_adjust parameter there.
+    seed: Seed to use for PRNGKey during sampling. For replicability run
+        this function and any other function that utilises predictions with the
+        same seed.
+
+  Returns:
+    Plot with Kernel density estimate smoothing showing prior and posterior
+    distributions for every parameter in the given media_mix_model.
+
+  Raises:
+    NotFittedModelError: media_mix_model has not yet been fit.
+    ValueError: A feature has been created without a well-defined prior.
+  """
+
+  (features, geo_level_features, channel_level_features, seasonal_features,
+   other_features) = _collect_features_for_prior_posterior_plot(
+       media_mix_model, selected_features)
+
+  number_of_subplots = int(
+      sum([
+          np.product(media_mix_model.trace[x].shape[1:]) for x in features
+      ]))
+
+  if not fig_size:
+    fig_size = (8, 1.5 * number_of_subplots)
+
+  fig = plt.figure(figsize=fig_size, constrained_layout=True)
+  gridspec_fig = matplotlib.gridspec.GridSpec(
+      nrows=number_of_subplots, ncols=1, figure=fig, hspace=0.1)
+
+  default_priors = {
+      **models._get_default_priors(),
+      **models._get_transform_default_priors()[media_mix_model.model_name]
+  }
+
+  kwargs_for_helper_function = {
+      "fig": fig,
+      "gridspec_fig": gridspec_fig,
+      "number_of_samples_for_prior": number_of_samples_for_prior,
+      "kde_bandwidth_adjust_for_posterior": kde_bandwidth_adjust_for_posterior,
+      "seed": seed,
+  }
+
+  i_ax = 0
+  for feature in (geo_level_features + channel_level_features +
+                  seasonal_features + other_features):
+    if feature not in features:
+      continue
+
+    if feature in media_mix_model.custom_priors:
+      prior_distribution = media_mix_model.custom_priors[feature]
+      if not isinstance(prior_distribution, numpyro.distributions.Distribution):
+        raise ValueError(f"{feature} cannot be plotted.")
+    elif feature in default_priors.keys():
+      prior_distribution = default_priors[feature]
+    elif feature in ("channel_coef_media", "coef_media"):
+      # We have to fill this in later since the prior varies by channel.
+      prior_distribution = None
+    else:
+      # This should never happen.
+      raise ValueError(f"{feature} has no prior specified.")
+    kwargs_for_helper_function["prior_distribution"] = prior_distribution
+
+    if feature == models._COEF_EXTRA_FEATURES:
+      for i_feature in range(media_mix_model.trace[feature].shape[1]):
+        for j_geo in range(media_mix_model.n_geos):
+          subplot_title = f"{feature} feature {i_feature}, geo {j_geo}"
+          if media_mix_model.n_geos == 1:
+            posterior_samples = np.array(
+                media_mix_model.trace[feature][:, i_feature])
+          else:
+            posterior_samples = np.array(
+                media_mix_model.trace[feature][:, i_feature, j_geo])
+          (fig, gridspec_fig,
+           i_ax) = _make_prior_and_posterior_subplot_for_one_feature(
+               posterior_samples=posterior_samples,
+               subplot_title=subplot_title,
+               i_ax=i_ax,
+               **kwargs_for_helper_function)
+
+    if feature in geo_level_features:
+      for i_geo in range(media_mix_model.n_geos):
+        subplot_title = f"{feature}, geo {i_geo}"
+        posterior_samples = np.array(media_mix_model.trace[feature][:, i_geo])
+        (fig, gridspec_fig,
+         i_ax) = _make_prior_and_posterior_subplot_for_one_feature(
+             posterior_samples=posterior_samples,
+             subplot_title=subplot_title,
+             i_ax=i_ax,
+             **kwargs_for_helper_function)
+
+    if feature in channel_level_features:
+      for i_channel in range(media_mix_model.n_media_channels):
+        subplot_title = f"{feature}, channel {i_channel}"
+        if feature in ("channel_coef_media", "coef_media"):
+          prior_distribution = numpyro.distributions.continuous.HalfNormal(
+              scale=jnp.squeeze(media_mix_model._media_prior[i_channel]))
+        posterior_samples = np.array(
+            jnp.squeeze(media_mix_model.trace[feature][:, i_channel]))
+        kwargs_for_helper_function["prior_distribution"] = prior_distribution
+        hyperprior = feature == "channel_coef_media"
+        (fig, gridspec_fig,
+         i_ax) = _make_prior_and_posterior_subplot_for_one_feature(
+             posterior_samples=posterior_samples,
+             subplot_title=subplot_title,
+             i_ax=i_ax,
+             hyperprior=hyperprior,
+             **kwargs_for_helper_function)
+
+    if feature in seasonal_features:
+      for i_season in range(media_mix_model._degrees_seasonality):
+        for j_season in range(2):
+          sin_or_cos = "sin" if j_season == 0 else "cos"
+          subplot_title = f"{feature}, seasonal mode {i_season}:{sin_or_cos}"
+          posterior_samples = np.array(media_mix_model.trace[feature][:,
+                                                                      i_season,
+                                                                      j_season])
+          (fig, gridspec_fig,
+           i_ax) = _make_prior_and_posterior_subplot_for_one_feature(
+               posterior_samples=posterior_samples,
+               subplot_title=subplot_title,
+               i_ax=i_ax,
+               **kwargs_for_helper_function)
+
+    if feature in other_features and feature != models._COEF_EXTRA_FEATURES:
+      subplot_title = f"{feature}"
+      posterior_samples = np.array(media_mix_model.trace[feature])
+      (fig, gridspec_fig,
+       i_ax) = _make_prior_and_posterior_subplot_for_one_feature(
+           posterior_samples=posterior_samples,
+           subplot_title=subplot_title,
+           i_ax=i_ax,
+           **kwargs_for_helper_function)
+  return fig
