@@ -1,4 +1,4 @@
-# Copyright 2022 Google LLC.
+# Copyright 2023 Google LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,8 +18,73 @@ from absl.testing import absltest
 from absl.testing import parameterized
 import jax.numpy as jnp
 import numpy as np
+import pandas as pd
 
 from lightweight_mmm import preprocessing
+from lightweight_mmm.core import core_utils
+
+
+_GEO_DATA_FOR_TESTS = [[[0.1, 0.5], [0.2, 0.4], [0.3, 0.7]],
+                       [[0.2, 0.5], [0.4, 0.6], [0.2, 0.9]],
+                       [[0.3, 0.5], [0.7, 0.8], [0.1, 0.1]],
+                       [[0.4, 0.5], [0.8, 0.5], [0, 0.2]],
+                       [[0.5, 0.5], [0.9, 0.6], [0, 0.5]]]
+_NATIONAL_DATA_FOR_TESTS = [[1, 2, 3, 5], [2, 3, 4, 5], [3, 4, 6, 5],
+                            [2, 5, 7, 5], [3, 7, 9, 5], [2, 6, 9, 5],
+                            [3, 5, 9, 5], [4, 8, 9, 5], [5, 7, 8, 5],
+                            [6, 9, 9, 5]]
+_NATIONAL_TARGET_DATA = [0.2, 0.4, 0.6, 0.8, 0.5, 0.7, 0.9, 1.0, 0.9, 1.2]
+_GEO_TARGET_DATA = [[0.2, 0.1, 1], [0.4, 0.2, 0.5], [0.6, 0.3, 1],
+                    [0.8, 0.4, 0], [1.0, 0.5, 0.2]]
+_NATIONAL_CORRELATION_MATRICES = [
+    pd.DataFrame(
+        data=[[1, 0.83381, 0.60244, np.nan, 0.81846],
+              [0.83381, 1, 0.86645, np.nan, 0.82736],
+              [0.60245, 0.86645, 1, np.nan, 0.77283],
+              [np.nan, np.nan, np.nan, np.nan, np.nan],
+              [0.81847, 0.82736, 0.77283, np.nan, 1]],
+        columns=["feature_0", "feature_1", "feature_2", "feature_3", "target"],
+        index=["feature_0", "feature_1", "feature_2", "feature_3", "target"],
+        dtype=float),
+]
+_GEO_CORRELATION_MATRICES = [
+    pd.DataFrame(
+        data=[[1, 0.97619, -0.97014, 1],
+              [0.97619, 1, -0.98650, 0.97610],
+              [-0.97014254, -0.9865005, 1, -0.97014,],
+              [1, 0.97619, -0.97014, 1]],
+        columns=["feature_0", "feature_1", "feature_2", "target"],
+        index=["feature_0", "feature_1", "feature_2", "target"],
+        dtype=float),
+    pd.DataFrame(
+        data=[[np.nan, np.nan, np.nan, np.nan],
+              [np.nan, 1, -0.46335, 0.31980],
+              [np.nan, -0.46335, 1, -0.51970],
+              [np.nan, 0.31980, -0.51970, 1]],
+        columns=["feature_0", "feature_1", "feature_2", "target"],
+        index=["feature_0", "feature_1", "feature_2", "target"],
+        dtype=float)
+]
+_NATIONAL_VARIANCES = pd.DataFrame(
+    data=[2.09, 4.44, 4.61, 0],
+    index=["feature_0", "feature_1", "feature_2", "feature_3"],
+    columns=["geo_0"],
+    dtype=float)
+_GEO_VARIANCES = pd.DataFrame(
+    data=[[0.02, 0], [0.068, 0.0176], [0.0136, 0.0896]],
+    index=["feature_0", "feature_1", "feature_2"],
+    columns=["geo_0", "geo_1"],
+    dtype=float)
+_NATIONAL_VIFS = pd.DataFrame(
+    data=[4.0491, 10.3485, 4.9505, 13.7372],
+    index=["feature_0", "feature_1", "feature_2", "feature_3"],
+    columns=["geo_0"],
+    dtype=float)
+_GEO_VIFS = pd.DataFrame(
+    data=[[22.1429, 36.8863], [48.5715, 1.2734], [38.8572, 1.2734]],
+    index=["feature_0", "feature_1", "feature_2"],
+    columns=["geo_0", "geo_1"],
+    dtype=float)
 
 
 class PreprocessingTest(parameterized.TestCase):
@@ -556,5 +621,272 @@ class PreprocessingTest(parameterized.TestCase):
     np.testing.assert_array_almost_equal(transformed_data,
                                          jnp.array(expected_transformed))
 
+  @parameterized.named_parameters([
+      dict(
+          testcase_name="national_data",
+          features=_NATIONAL_DATA_FOR_TESTS,
+          target=_NATIONAL_TARGET_DATA,
+          expected_correlations=_NATIONAL_CORRELATION_MATRICES,
+      ),
+      dict(
+          testcase_name="geo_data",
+          features=_GEO_DATA_FOR_TESTS,
+          target=_GEO_TARGET_DATA,
+          expected_correlations=_GEO_CORRELATION_MATRICES,)
+  ])
+  def test_compute_correlations_returns_expected_values(
+      self, features, target, expected_correlations):
+    features = jnp.array(features)
+    target = jnp.array(target)
+    feature_names = [f"feature_{i}" for i in range(features.shape[1])]
+
+    correlations = preprocessing._compute_correlations(
+        features=features, target=target, feature_names=feature_names)
+
+    for i, expected_correlation in enumerate(expected_correlations):
+      pd.testing.assert_frame_equal(
+          correlations[i], expected_correlation, atol=1e-3, check_dtype=False)
+
+  @parameterized.named_parameters([
+      dict(
+          testcase_name="1_dimensional_target",
+          features=np.ones([5, 2, 3]),
+          target=np.zeros(5),
+          expected_message=(r"Incompatible shapes between features \(5, 2, 3\) "
+                            r"and target \(5,\)\.")
+      ),
+      dict(
+          testcase_name="2_dimensional_target",
+          features=np.ones([10, 5]),
+          target=np.zeros([5, 5]),
+          expected_message=(r"Incompatible shapes between features \(10, 5\) "
+                            r"and target \(5, 5\)\."))
+  ])
+  def test_compute_correlations_raises_value_error(self, features, target,
+                                                   expected_message):
+    feature_names = [f"feature_{i}" for i in range(features.shape[1])]
+
+    with self.assertRaisesRegex(ValueError, expected_message):
+      preprocessing._compute_correlations(
+          features=features, target=target, feature_names=feature_names)
+
+  @parameterized.named_parameters([
+      dict(
+          testcase_name="national_data",
+          features=_NATIONAL_DATA_FOR_TESTS,
+          target=_NATIONAL_TARGET_DATA,
+          expected_correlations=_NATIONAL_CORRELATION_MATRICES),
+      dict(
+          testcase_name="geo_data",
+          features=_GEO_DATA_FOR_TESTS,
+          target=_GEO_TARGET_DATA,
+          expected_correlations=_GEO_CORRELATION_MATRICES),
+      ])
+  def test_check_data_quality_with_extra_features(self, features, target,
+                                                  expected_correlations):
+    media_data = jnp.array(features)[:, :2]
+    costs = np.ones(media_data.shape[1])
+    extra_features = jnp.array(features)[:, 2:]
+    extra_features_transformer = {
+        "feature_2": "extra_feature_0",
+        "feature_3": "extra_feature_1",
+    }
+    updated_expected_correlations = [
+        x.rename(
+            index=extra_features_transformer,
+            columns=extra_features_transformer) for x in expected_correlations
+    ]
+
+    correlations, _, _, _ = preprocessing.check_data_quality(
+        media_data=media_data,
+        target_data=jnp.array(target),
+        cost_data=costs,
+        extra_features_data=extra_features)
+
+    for i, expected_correlation in enumerate(updated_expected_correlations):
+      pd.testing.assert_frame_equal(
+          correlations[i], expected_correlation, atol=1e-3, check_dtype=False)
+
+  @parameterized.named_parameters([
+      dict(
+          testcase_name="national_data",
+          features=_NATIONAL_DATA_FOR_TESTS,
+          expected_variances=_NATIONAL_VARIANCES,
+      ),
+      dict(
+          testcase_name="geo_data",
+          features=_GEO_DATA_FOR_TESTS,
+          expected_variances=_GEO_VARIANCES,
+      )
+  ])
+  def test_compute_variances_returns_expected_values(self, features,
+                                                     expected_variances):
+    features = jnp.array(features)
+    feature_names = [f"feature_{i}" for i in range(features.shape[1])]
+    geo_names = ["geo_0", "geo_1"] if features.ndim == 3 else ["geo_0"]
+
+    variances = preprocessing._compute_variances(
+        features=features, feature_names=feature_names, geo_names=geo_names)
+
+    pd.testing.assert_frame_equal(
+        variances, expected_variances, atol=1e-3, check_dtype=False)
+
+  def test_check_data_quality_raises_error_on_media_channel_name_mismatch(self):
+    expected_message = ("Number of channels in media_data does not match "
+                        "length of channel_names")
+    with self.assertRaisesRegex(ValueError, expected_message):
+      preprocessing.check_data_quality(
+          media_data=jnp.ones([3, 3]),
+          target_data=jnp.ones(3),
+          channel_names=["channel_one", "channel_two"],
+          cost_data=jnp.ones(3))
+
+  def test_check_data_quality_raises_error_on_extra_feature_name_mismatch(self):
+    expected_message = ("Number of features in extra_features_data does not "
+                        "match length of extra_features")
+    with self.assertRaisesRegex(ValueError, expected_message):
+      preprocessing.check_data_quality(
+          media_data=jnp.ones([3, 3]),
+          target_data=jnp.ones(3),
+          cost_data=jnp.ones(3),
+          extra_features_data=jnp.ones([3, 4]),
+          channel_names=["channel_one", "channel_two", "channel_three"],
+          extra_features_names=["extra_feature_0", "extra_feature_1"])
+
+  def test_check_data_quality_raises_error_on_cost_data_mismatch(self):
+    expected_message = ("Number of channels in cost_data does not match "
+                        "length of channel_names")
+    with self.assertRaisesRegex(ValueError, expected_message):
+      preprocessing.check_data_quality(
+          media_data=jnp.ones([3, 3]),
+          target_data=jnp.ones(3),
+          channel_names=["channel_one", "channel_two", "channel_three"],
+          cost_data=jnp.ones(5))
+
+  @parameterized.product(
+      (dict(
+          costs=np.arange(1, 10),
+          expected_spend_fractions=np.arange(1, 10) / np.arange(1, 10).sum()),
+       dict(costs=np.ones(5), expected_spend_fractions=np.ones(5) / 5.)),
+      (dict(channel_names=None),
+       dict(channel_names=[f"channel_{x}" for x in "ABCDEFGHIJ"])))
+  def test_compute_spend_fraction_results_are_correct(self, costs,
+                                                      expected_spend_fractions,
+                                                      channel_names):
+    if channel_names is not None:
+      channel_names = channel_names[:len(costs)]
+    expected_output = pd.DataFrame(
+        expected_spend_fractions,
+        index=channel_names,
+        columns=["fraction of spend"])
+
+    spend_fractions = preprocessing._compute_spend_fractions(
+        costs, channel_names)
+
+    pd.testing.assert_frame_equal(spend_fractions, expected_output, atol=1e-3)
+
+  @parameterized.named_parameters([
+      dict(testcase_name="single_zero", costs=np.arange(10)),
+      dict(testcase_name="all_zeros", costs=np.zeros(10)),
+      dict(testcase_name="negative_number_and_zero", costs=np.arange(-1, 10)),
+  ])
+  def test_compute_spend_fraction_raises_error_on_non_positive_costs(
+      self, costs):
+    expected_message = ("Values in cost_data must all be positive")
+    with self.assertRaisesRegex(ValueError, expected_message):
+      preprocessing._compute_spend_fractions(costs)
+
+  @parameterized.named_parameters([
+      dict(
+          testcase_name="national_data",
+          features=_NATIONAL_DATA_FOR_TESTS,
+          expected_vifs=_NATIONAL_VIFS,
+      ),
+      dict(
+          testcase_name="geo_data",
+          features=_GEO_DATA_FOR_TESTS,
+          expected_vifs=_GEO_VIFS,
+      )
+  ])
+  def test_compute_vifs_returns_expected_values(self, features, expected_vifs):
+    features = jnp.array(features)
+    feature_names = [f"feature_{i}" for i in range(features.shape[1])]
+    geo_names = ["geo_0", "geo_1"] if features.ndim == 3 else ["geo_0"]
+
+    vifs = preprocessing._compute_variance_inflation_factors(
+        features=features, feature_names=feature_names, geo_names=geo_names)
+
+    pd.testing.assert_frame_equal(
+        vifs, expected_vifs, atol=1e-3, check_dtype=False)
+
+  def test_extreme_values_for_compute_vifs(self):
+    df = pd.DataFrame(
+        data={
+            "column_A": np.arange(25),
+            "column_B": np.arange(25)**0.5 + 2,
+            "column_C": np.arange(25)**0.25 - 5,
+            "all_ones": np.ones(25),
+            "all_zeros": np.zeros(25)
+        })
+    df["linear_transform_of_column_A"] = 30 - df["column_A"]
+    df["copy_of_column_B"] = df["column_B"]
+
+    vifs = preprocessing._compute_variance_inflation_factors(
+        features=df.values, feature_names=df.columns, geo_names=["a_geo"])
+    expected_vifs = pd.DataFrame(
+        data=[np.inf, np.inf, 57.8253, 0, np.nan, np.inf, np.inf],
+        columns=["a_geo"],
+        index=[
+            "column_A", "column_B", "column_C", "all_ones", "all_zeros",
+            "linear_transform_of_column_A", "copy_of_column_B"
+        ])
+
+    pd.testing.assert_frame_equal(
+        vifs, expected_vifs, atol=1e-3, check_dtype=False)
+
+  @parameterized.named_parameters([
+      dict(
+          testcase_name="national_data",
+          features=_NATIONAL_DATA_FOR_TESTS,
+          geo_names=["geo_0", "geo_1"],
+      ),
+      dict(
+          testcase_name="geo_data",
+          features=_GEO_DATA_FOR_TESTS,
+          geo_names=["geo_0"],
+      )
+  ])
+  def test_compute_vifs_raises_error_for_incorrect_number_of_geo_names(
+      self, features, geo_names):
+    features = jnp.array(features)
+    feature_names = [f"feature_{i}" for i in range(features.shape[1])]
+    expected_message = ("The number of geos in features does not match the "
+                        "length of geo_names")
+    with self.assertRaisesRegex(ValueError, expected_message):
+      preprocessing._compute_variance_inflation_factors(
+          features=features, feature_names=feature_names, geo_names=geo_names)
+
+  @parameterized.named_parameters([
+      dict(
+          testcase_name="national_data",
+          features=_NATIONAL_DATA_FOR_TESTS,
+          geo_names=["geo_0", "geo_1"],
+      ),
+      dict(
+          testcase_name="geo_data",
+          features=_GEO_DATA_FOR_TESTS,
+          geo_names=["geo_0"],
+      )
+  ])
+  def test_compute_variances_raises_error_for_incorrect_number_of_geo_names(
+      self, features, geo_names):
+    features = jnp.array(features)
+    feature_names = [f"feature_{i}" for i in range(features.shape[1])]
+    expected_message = ("The number of geos in features does not match the "
+                        "length of geo_names")
+    with self.assertRaisesRegex(ValueError, expected_message):
+      preprocessing._compute_variances(
+          features=features, feature_names=feature_names, geo_names=geo_names)
+
 if __name__ == "__main__":
   absltest.main()
